
# Some of my (not yet working) experiments in reading the encoder disc

#-----------------------------------------------------------------------
# Encoder read via "State Machine", four states per tooth
# Code exceeds 32 command lines
# and so can't be executed by Raspberry Pico
# So its just how it could be....
# ...I am working on it...
#-----------------------------------------------------------------------

@asm_pio(autopush=True, push_thresh=32)
def encoder():
    label("start")
    set(y,0)
    set(x,0)
    label("state00_plus")
    mov(y, invert(x))           # Increment X
    jmp(y_dec, "state00_plus_1")
    label("state00_plus_1")
    mov(x, invert(y))
    jmp("state00")
    label("state00_minus")
    jmp(x_dec, "state00")       # Decrement X
    label("state00")
    in_(pins, 1)                # move state of in_base to isr
    mov(y, isr)                 # move bits from ISR to Y
    jmp(y, "state01_plus")      # State changed to 01
    jmp(pin,"state10_minus")    # State changed to 10
    jmp("state00")              # nothing changed, back to state 00

    label("state01_plus")
    mov(y, invert(x))           # Increment X
    jmp(y_dec, "state01_plus_1")
    label("state01_plus_1")
    mov(x, invert(y))
    jmp("state01")
    label("state01_minus")
    jmp(x_dec, "state01")       # Decrement X
    label("state01")
    in_(pins, 1)                # move state of in_base to isr
    mov(y, isr)                 # move bits from ISR to Y
    jmp(not_y, "state00_minus") # State changed to 00
    jmp(pin,"state11_plus")     # State changed to state 11
    jmp("state01")              # nothing changed, back to state 00

    label("state11_plus")
    mov(y, invert(x))           # Increment X
    jmp(y_dec, "state11_plus_1")
    label("state11_plus_1")
    mov(x, invert(y))
    jmp("state11")
    label("state11_minus")
    jmp(x_dec, "state11")       # Decrement X
    label("state11")
    in_(pins, 1)                # move state of in_base to isr
    mov(y, isr)                 # move bits from ISR to Y
    jmp(not_y, "state10_plus")  # State changed to 10
    jmp(pin,"state11")          # nothing changed, back to state 11
    jmp("state01_minus")        # State changed to state 01

    label("state10_plus")
    mov(y, invert(x))           # Increment X
    jmp(y_dec, "state10_plus_1")
    label("state10_plus_1")
    mov(x, invert(y))
    jmp("state10")
    label("state10_minus")
    jmp(x_dec, "state10")       # Decrement X
    label("state10")
    in_(pins, 1)                # move state of in_base to isr
    mov(y, isr)                 # move bits from ISR to Y
    jmp(y, "state11_minus")     # State changed to 11
    jmp(pin,"state10")          # nothing changed, back to state 11
    jmp("state00_plus")         # State changed to state 01

    wrap()

sm1 = StateMachine(1, encoder, freq=125_000_000, in_base=Pin(HALL_1_PIN), jmp_pin=Pin(HALL_2_PIN))
sm1.active(1) # Start the mini program





#-----------------------------------------------------------------------
# Doing it all with IRQ (slower due to latency):
#-----------------------------------------------------------------------

from   machine import Pin
from   rp2     import asm_pio, PIO, StateMachine
import time

ENCODER_PINS = [0, 1]

Pin(ENCODER_PINS[0], Pin.IN, Pin.PULL_UP)
Pin(ENCODER_PINS[1], Pin.IN, Pin.PULL_UP)

@asm_pio()
def ReadQuadratureSwitchChanges(in_init=[PIO.IN_LOW] * 2, rx_join=True):
    mov(y, invert(null))

    label("loop")
    mov(isr, null)
    in_(pins, 2)
    mov(x, isr)
    jmp(x_not_y, "changed")
    jmp("loop")

    label("changed")
    mov(y, x)
    push(block)
    irq(0)
    jmp("loop")

def A(was, now) : return (was << 3) | (now << 1)
def B(was, now) : return (was << 2) | (now << 0)

incs = [0] * 16
incs[ A(0,1) + B(0,0) ] = +1
incs[ A(1,0) + B(1,1) ] = +1
incs[ A(0,0) + B(1,0) ] = +1
incs[ A(1,1) + B(0,1) ] = +1
incs[ A(0,1) + B(1,1) ] = -1
incs[ A(1,0) + B(0,0) ] = -1
incs[ A(0,0) + B(0,1) ] = -1
incs[ A(1,1) + B(1,0) ] = -1

bits = -1
count = 0

def Irq(sm):
    global bits
    global count
    while sm.rx_fifo():
        if bits < 0:
            bits = sm.get()
            print("--{:02b} {}".format(bits, count))
        else:
            bits = ((bits & 3) << 2) | sm.get()
            count = count + incs[bits]
            print("{:04b} {}".format(bits, count))

sm = StateMachine(0, ReadQuadratureSwitchChanges, in_base=Pin(ENCODER_PINS[0]))
sm.irq(Irq)
sm.active(1)

while True:
    time.sleep(10)

